
% use vectors to store the upper-triangle matrices.
% 
% By Wen Huang

logind = find(triu(ones(1400, 1400), 1) ~= 0);
MTQNU_v = MTQNUC(logind);
MTQNUS_v = MTQNUS(logind);
MIQNU_v = MIQNUC(logind);
MTDP2U_v = MTDP2U(logind);
MTDP4U_v = MTDP4U(logind);
MTDP8U_v = MTDP8U(logind);
MTDP16U_v = MTDP16U(logind);

[sum(MTQNU_v) / length(MTQNU_v),
sum(MTQNUS_v) / length(MTQNUS_v),
sum(MTDP16U_v) / length(MTDP16U_v),
sum(MTDP8U_v) / length(MTDP8U_v),
sum(MTDP4U_v) / length(MTDP4U_v),
sum(MTDP2U_v) / length(MTDP2U_v)]

logind = find(triu(ones(1907, 1907), 1) ~= 0);
FTQNU_v = FTQNUC(logind);
FTQNUS_v = FTQNUS(logind);
FIQNU_v = FIQNUC(logind);
FTDP2U_v = FTDP2U(logind);
FTDP4U_v = FTDP4U(logind);
FTDP8U_v = FTDP8U(logind);
FTDP16U_v = FTDP16U(logind);

[sum(FTQNU_v) / length(FTQNU_v),
sum(FTQNUS_v) / length(FTQNUS_v),
sum(FTDP16U_v) / length(FTDP16U_v),
sum(FTDP8U_v) / length(FTDP8U_v),
sum(FTDP4U_v) / length(FTDP4U_v),
sum(FTDP2U_v) / length(FTDP2U_v)]

% Plot histograms of computational time of FLAVIA and MPEG7.
% and the computational time per initial condition for both data sets.
% 
% By Wen Huang

figure(1);clf
    set(0,'defaultaxesfontsize',14, ...
       'defaultaxeslinewidth',0.7, ...
       'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
    set(0,'defaultlinemarkersize',10)
    subplot(1, 2, 1);
    Y = zeros(length(MTQNU_v), 5);
    Y(:, 1) = MTQNU_v';
    Y(:, 2) = MTDP2U_v';
    Y(:, 3) = MTDP4U_v';
    Y(:, 4) = MTDP8U_v';
    Y(:, 5) = MTDP16U_v';
    hist(Y, 0 : 0.02 : 1);
fprintf('MPEG7:\nN/2: %f, %d\n', sum(MTQNU_v > min(MTDP2U_v)) / length(MTQNU_v), sum(MTQNU_v > min(MTDP2U_v)))
fprintf('N/4: %f, %d\n', sum(MTQNU_v > min(MTDP4U_v)) / length(MTQNU_v), sum(MTQNU_v > min(MTDP4U_v)))
fprintf('N/8: %f, %d\n', sum(MTQNU_v > min(MTDP8U_v)) / length(MTQNU_v), sum(MTQNU_v > min(MTDP8U_v)))
fprintf('N/16: %f, %d\n', sum(MTQNU_v > min(MTDP16U_v)) / length(MTQNU_v), sum(MTQNU_v > min(MTDP16U_v)))
    
    legend('LRBFGS', 'CD1H: N/2', 'CD1H: N/4', 'CD1H: N/8', 'CD1H: N/16');
    xlim([0, 1]);
%     title('1NN results of Flavia leaf dataset');
    title('MPEG-7');
    xlabel('time(seconds)');
    ylabel('number');
    legend('boxoff');
    
    subplot(1, 2, 2);
    Y = zeros(length(FTQNU_v), 5);
    Y(:, 1) = FTQNU_v';
    Y(:, 2) = FTDP2U_v';
    Y(:, 3) = FTDP4U_v';
    Y(:, 4) = FTDP8U_v';
    Y(:, 5) = FTDP16U_v';
    hist(Y, 0 : 0.02 : 1);
fprintf('FLAVIA:\nN/2: %f, %d\n', sum(FTQNU_v > min(FTDP2U_v)) / length(FTQNU_v), sum(FTQNU_v > min(FTDP2U_v)))
fprintf('N/4: %f, %d\n', sum(FTQNU_v > min(FTDP4U_v)) / length(FTQNU_v), sum(FTQNU_v > min(FTDP4U_v)))
fprintf('N/8: %f, %d\n', sum(FTQNU_v > min(FTDP8U_v)) / length(FTQNU_v), sum(FTQNU_v > min(FTDP8U_v)))
fprintf('N/16: %f, %d\n', sum(FTQNU_v > min(FTDP16U_v)) / length(FTQNU_v), sum(FTQNU_v > min(FTDP16U_v)))
    
    legend('LRBFGS', 'CD1H: N/2', 'CD1H: N/4', 'CD1H: N/8', 'CD1H: N/16');
    xlim([0, 1]);
%     title('1NN results of Flavia leaf dataset');
    title('Flavia');
    xlabel('time(seconds)');
    ylabel('number');
    legend('boxoff');
    
    
figure(2);clf
    set(0,'defaultaxesfontsize',14, ...
       'defaultaxeslinewidth',0.7, ...
       'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
    set(0,'defaultlinemarkersize',10)
    subplot(1, 2, 1);
    Y = zeros(length(MTQNU_v), 2);
    Y(:, 1) = MTQNU_v';
    Y(:, 2) = MTQNU_v' ./ MIQNU_v';
    
    [n1, xout1] = hist(Y(:, 1), 0.005 : 0.01 : 0.6);
    b1 = bar(xout1, n1);
%     ybs = 3e4; ybe = 10e4;
%     b1 = BarPlotBreak2(xout1, n1, ybs, ybe, 'Patch');
    set(b1,'FaceColor','b','EdgeColor','w')
    hold on
    [n2, xout2] = hist(Y(:, 2), 0.005 : 0.01 : 0.6);
%     b2 = bar(xout2, n2);
    ybs = 2e4; ybe = 10.5e4;
    b2 = BarPlotBreak(xout2, n2, ybs, ybe, 'Patch');
    set(b2,'FaceColor','r','EdgeColor','w')
    xlim([0, 0.6]);
    
    legend('time', 'PIC');
    title('MPEG-7');
    xlabel('time(seconds)');
    ylabel('number');
    
    subplot(1, 2, 2);
    Y = zeros(length(FTQNU_v), 2);
    Y(:, 1) = FTQNU_v';
    Y(:, 2) = FTQNU_v' ./ FIQNU_v';

    [n1, xout1] = hist(Y(:, 1), 0.005 : 0.01 : 0.4);
    b1 = bar(xout1, n1);
    set(b1,'FaceColor','b','EdgeColor','w')
    hold on
    [n2, xout2] = hist(Y(:, 2), 0.005 : 0.01 : 0.4);
%     b2 = bar(xout2, n2);
    ybs = 7e4; ybe = 3.9e5;
    b2 = BarPlotBreak(xout2, n2, ybs, ybe, 'Patch');
    set(b2,'FaceColor','r','EdgeColor','w')
    xlim([0, 0.4]);
    
%     hist(Y, 0 : 0.05 : 1.8);
% %     h = findobj(gca,'Type','patch');
% %     set(h,'FaceColor','r','EdgeColor','r')
%     xlim([0, 1.8]);
% %     title('1NN results of Flavia leaf dataset');
    legend('time', 'PIC');
    title('Flavia');
    xlabel('time(seconds)');
    ylabel('number');
    
figure(3);clf
    set(0,'defaultaxesfontsize',12, ...
       'defaultaxeslinewidth',0.7, ...
       'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
    set(0,'defaultlinemarkersize',10)
    subplot(2, 2, 1);
    Y = zeros(length(MTQNU_v), 2);
    Y(:, 1) = MTQNU_v';
    Y(:, 2) = MTQNU_v' ./ MIQNU_v';
    
%     hist(Y,  0 : 0.05 : 2.5);
%     xlim([0, 2.5]);
% %     title('1NN results of Flavia leaf dataset');
    
    [n1, xout1] = hist(Y(:, 1), 0.005 : 0.01 : 0.6);
    b1 = bar(xout1, n1);
    set(b1,'FaceColor','b','EdgeColor','w')
    xlim([0, 0.6]);
    
    MPEG7TIMSTD = std(Y(:, 1))
    MPEG7PICSTD = std(Y(:, 2))
    
    legend('time');
    title('MPEG-7');
    xlabel('time(seconds)');
    ylabel('number');
    
    subplot(2, 2, 2);
    Y = zeros(length(FTQNU_v), 2);
    Y(:, 1) = FTQNU_v';
    Y(:, 2) = FTQNU_v' ./ FIQNU_v';

    [n1, xout1] = hist(Y(:, 1), 0.005 : 0.01 : 0.4);
    b1 = bar(xout1, n1);
    set(b1,'FaceColor','b','EdgeColor','w')
    xlim([0, 0.4]);
    
    FLAVIATIMSTD = std(Y(:, 1))
    FLAVIAPICSTD = std(Y(:, 2))
    
%     hist(Y, 0 : 0.05 : 1.8);
% %     h = findobj(gca,'Type','patch');
% %     set(h,'FaceColor','r','EdgeColor','r')
%     xlim([0, 1.8]);
% %     title('1NN results of Flavia leaf dataset');
    legend('time');
    title('Flavia');
    xlabel('time(seconds)');
    ylabel('number');
    
    
    
    subplot(2, 2, 3);
    Y = zeros(length(MTQNU_v), 2);
    Y(:, 1) = MTQNU_v';
    Y(:, 2) = MTQNU_v' ./ MIQNU_v';
    
    [n2, xout2] = hist(Y(:, 2), 0.0005 : 0.001 : 0.03);
    b2 = bar(xout2, n2);
    set(b2,'FaceColor','r','EdgeColor','w')
    xlim([0, 0.03]);
    
    legend('PIC');
    title('MPEG-7');
    xlabel('time(seconds)');
    ylabel('number');
    
    subplot(2, 2, 4);
    Y = zeros(length(FTQNU_v), 2);
    Y(:, 1) = FTQNU_v';
    Y(:, 2) = FTQNU_v' ./ FIQNU_v';
    
    [n2, xout2] = hist(Y(:, 2), 0.0005 : 0.001 : 0.02);
    b2 = bar(xout2, n2);
    set(b2,'FaceColor','r','EdgeColor','w')
    xlim([0, 0.02]);
    
    legend('PIC');
    title('Flavia');
    xlabel('time(seconds)');
    ylabel('number');
    