% f = fourierShift(f, d, dt)
% 
% FFT based signal time shift
% 
% Input:
%   f  - row vector of signal to be shifted. If f is a matrix each row is
%        shifted.
%   d  - desired delay
%   dt - sample spacing
% 
% Output:
%   f  - time shifted signal
% 
function f = fourierShift(f, d, dt)
    colvec = 0;
    
    if isvector(f) && size(f,1)~=1
        colvec = 1;
        f = f';
    end
    
    [m,n] = size(f);
    if isreal(f)
        realVal = 1;
    else
        realVal = 0;
    end
    
    f = fft(f,[],2);
    f = fftshift(f,2);
    
    w = (-ceil((n-1)/2):floor((n-1)/2))/dt/n;
    f = f.*exp(-2*pi*1i*d*w(ones(1,m),:));
    
    f = ifftshift(f,2);
    f = ifft(f,[],2);
    
    if realVal
        f = real(f);
    end
    
    dn = ceil(abs(d)/dt);
    ix = (1:n);
    if d>0
        f = f.*(ix(ones(1,m),:)>dn);
    elseif d<0
        
        f = f.*(ix(ones(1,m),:)<n-dn);
    end
    
    if colvec
        f = f';
    end
end