function test_MIMO_CPP()
% clear all;clc;close all;

    % Brother's parameters
    Tw = 0.2;
    W = 30; % 30kHz;
    % dt = 8.3333e-6

    % lag milisecond pulse
    % valid lags are anything between -T and T
    % How many lags? (open question)

    % Increasing number of signals causes break down because
    % of interference
    nb = 4; ns = 2;

    delta = 2;

    % Options
    opt.Display = 1;

    fprintf('test for MIMO problem. \n');
    % Compute time samples
    Tw = Tw/2;
    dt = 1/(4*W);
    xt = -Tw:dt:Tw;

    % Compute zero lag pswf functions
    fn = sprintf('pswf_func_%f_%d_%d.mat', Tw, W, nb);
    if exist(fn, 'file')
        load(fn)
    else
        p0 = pswf(xt/Tw, Tw, W, nb).';
        save(fn, 'p0');
    end


    % Compute correlation matrix for each lag
    % Non-periodic soln
    % If you give it a shift longer than your signal than you get zero
    % zero pad to extend length for applying the biggest lag
    
    lag = -2*Tw:dt:2*Tw;
    maxLag = max(lag)/dt;
    pad = ceil(maxLag);
    p0 = [p0; zeros(pad, nb)];
    
    nlags = length(lag);
    C = zeros(nb, nb, nlags);
    for l = 1 : nlags
        pl = fourierShift(p0.', lag(l), dt).';
        C(:, :, l) = p0' * pl;
    end
    flag = sprintf('lags_%f_%d_%d.mat', 2*Tw, W, nb);
    save(flag, 'lag');

    fhandle = @(x)f(x, C, lag, delta);
    gfhandle = @(x)gf(x, C, lag, delta);
    Hesshandle = @(x, eta)Hess(x, eta, C, lag, delta);

    SolverParams.method = 'LRBFGS';
    SolverParams.IsCheckParams = 1;
    SolverParams.DEBUG = 2;
    SolverParams.Max_Iteration = 1000;
%     SolverParams.IsCheckGradHess = 1;

%     ManiParams.IsCheckParams = 1;
%     ManiParams(1).name = 'Sphere';
%     ManiParams(1).numofmani = ns;
%     ManiParams(1).n = nb;
%     ManiParams(1).ParamSet = 1;

    ManiParams.IsCheckParams = 1;
    ManiParams(1).name = 'Oblique';
    ManiParams(1).numofmani = 1;
    ManiParams(1).n = nb;
    ManiParams(1).m = ns;
    ManiParams(1).ParamSet = 1;
    
    HasHHR = 0;

    % Initial Condtions
    x1 = randn(nb, ns);
    [x1, ~] = qr(x1, 0);
%     x1 = reshape(x1, nb*ns, 1);
    
    H = zeros(nb, nb, ns);
    for i = 1 : ns
        H(:, :, i) = eye(nb);
    %         B(:, :, i) = eye(nb);
    end

    initialX.main = x1;

    output = DriverOPT(fhandle, gfhandle, Hesshandle, SolverParams, ManiParams, HasHHR, initialX);  
end


    %% Objects
function [output, x] = f(x, C, lag, delta)
    % Cost Function
    [nb,  ns] = size(x.main);
    N = size(C, 3);
    I1 = 0;
    b = lag(N);
    a = lag(1);
    
    x.UtCU = zeros(ns, ns, N);
    
    % Neighborhood around tau = 0
    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    for i = 1 : dl
        x.UtCU(:, :, i) = x.main' * C(:, :, i) * x.main;
        f = norm(x.UtCU(:, :, i),'fro')^2;
        
        if(i == 1)
            I1 = I1 + ((b - a) / (N - 1)) * (f / 2);
        else
            I1 = I1 + ((b - a) / (N - 1)) * f;
        end
    end
    
    for i = dl : du
        x.UtCU(:, :, i) = x.main' * C(:, :, i) * x.main;
        fO = norm((triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1)),'fro')^2;
        
        I1 = I1 + ((b - a) / (N - 1)) * fO;
    end
    
    for i = du : N
        x.UtCU(:, :, i) = x.main' * C(:, :, i) * x.main;
        f = norm(x.UtCU(:, :, i),'fro')^2;
        
        if(i == N)
            I1 = I1 + ((b - a) / (N - 1)) * (f / 2);
        else
            I1 = I1 + ((b - a) / (N - 1)) * f;
        end
    end
    output = I1;
end

function [output, x] = gf(x, C, lag, delta)
    % Gradient
    [nb, ns] = size(x.main);
    N = size(C, 3);
    Q1 = zeros(nb, ns);
    b = lag(N);
    a = lag(1);
    
    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    x.CU = zeros(nb, ns, N);
    x.CtU = zeros(nb, ns, N);
    
    if(~isfield(x, 'UtCU'))
        for i = 1 : N
            x.UtCU(:, :, i) = x.main' * C(:, :, i) * x.main;
        end
    end
    
    for i = 1 : dl
        x.CU(:, :, i) = C(:, :, i) * x.main;
        x.CtU(:, :, i) = C(:, :, i)' * x.main;
        gf = (x.CU(:, :, i) * x.UtCU(:, :, i)') + (x.CtU(:, :, i) * x.UtCU(:, :, i));
        
        if(i == 1)
            Q1 = Q1 + ((b - a) / (N - 1)) * (gf / 2);
        else
            Q1 = Q1 + ((b - a) / (N - 1)) * gf;
        end
    end
    
    for i = dl : du
        x.CU(:, :, i) = C(:, :, i) * x.main;
        x.CtU(:, :, i) = C(:, :, i)' * x.main;
        gfO = (x.CU(:, :, i) * (triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1))') + (x.CtU(:, :, i) * (triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1)));
        
        Q1 = Q1 + ((b - a) / (N - 1)) * gfO;
    end
    
    for i = du : N
        x.CU(:, :, i) = C(:, :, i) * x.main;
        x.CtU(:, :, i) = C(:, :, i)' * x.main;
        gf = (x.CU(:, :, i) * x.UtCU(:, :, i)') + (x.CtU(:, :, i) * x.UtCU(:, :, i));
        
        if(i == N)
            Q1 = Q1 + ((b - a) / (N - 1)) * (gf / 2);
        else
            Q1 = Q1 + ((b - a) / (N - 1)) * gf;
        end
    end

    Q1 = 2 * Q1;
    output.main = Q1 - x.main * diag(diag(x.main' * Q1));
end

function [output, x] = Hess(x, eta, C, lag, delta)
    % Hessian
    [nb, ns] = size(x.main);
    N = size(C, 3);
    Q1 = zeros(nb, ns);
    Q2 = zeros(nb, ns);
    b = lag(N);
    a = lag(1);

    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    if(~isfield(x, 'CU'))
        for i = 1 : N
            x.CU(:, :, i) = C(:, :, i) * x.main;
        end
    end
    
    if(~isfield(x, 'UtCU'))
        for i = 1 : N
            x.UtCU(:, :, i) = x.main' * C(:, :, i) * x.main;
        end
    end
    
    if(~isfield(x, 'CtU'))
        for i = 1 : N
            x.CtU(:, :, i) = C(:, :, i)' * x.main;
        end
    end
    
    for i = 1 : dl
        gf = (x.CU(:, :, i) * x.UtCU(:, :, i)') + (x.CtU(:, :, i) * x.UtCU(:, :, i));
       
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * x.CU(:,:,i);
        tmp7 = x.main' * tmp4;
        
        Dgf = (tmp4 * x.UtCU(:, :, i)') + (x.CU(:, :, i) * tmp6') + (x.CU(:, :, i) * tmp7') + ...
            (tmp5 * x.UtCU(:, :, i)) + (x.CtU(:, :, i) * tmp6) + (x.CtU(:, :, i) * tmp7);
        
        if(i == 1)
            Q1 = Q1 + ((b - a) / (N-1)) * (gf / 2);
            Q2 = Q2 + ((b - a) / (N-1)) * (Dgf / 2);
        else
            Q1 = Q1 + ((b - a) / (N-1)) * gf;
            Q2 = Q2 + ((b - a) / (N-1)) * Dgf;
        end
    end
    
    for i = dl : du
        gfO = (x.CU(:, :, i) * (triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1))') + (x.CtU(:, :, i) * (triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1)));
        
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * x.CU(:,:,i);
        tmp7 = x.main' * tmp4;
        
        DgfO = (tmp4 * (triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1))') + (x.CU(:, :, i) * (triu(tmp6,1) + tril(tmp6,-1))') + ...
            (x.CU(:, :, i) * (triu(tmp7,1) + tril(tmp7,-1))') + (tmp5 * (triu(x.UtCU(:, :, i),1) + tril(x.UtCU(:, :, i),-1))) + ...
            (x.CtU(:, :, i) * (triu(tmp6,1) + tril(tmp6,-1))) + (x.CtU(:, :, i) * (triu(tmp7,1) + tril(tmp7,-1)));
        
        Q1 = Q1 + ((b - a) / (N-1)) * gfO;
        Q2 = Q2 + ((b - a) / (N-1)) * DgfO;
    end
    
    for i = du : N
        gf = (x.CU(:, :, i) * x.UtCU(:, :, i)') + (x.CtU(:, :, i) * x.UtCU(:, :, i));
       
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * x.CU(:,:,i);
        tmp7 = x.main' * tmp4;
        
        Dgf = (tmp4 * x.UtCU(:, :, i)') + (x.CU(:, :, i) * tmp6') + (x.CU(:, :, i) * tmp7') + ...
            (tmp5 * x.UtCU(:, :, i)) + (x.CtU(:, :, i) * tmp6) + (x.CtU(:, :, i) * tmp7);
        
        if(i == N)
            Q1 = Q1 + ((b - a) / (N-1)) * (gf / 2);
            Q2 = Q2 + ((b - a) / (N-1)) * (Dgf / 2);
        else
            Q1 = Q1 + ((b - a) / (N-1)) * gf;
            Q2 = Q2 + ((b - a) / (N-1)) * Dgf;
        end
    end 
    
    Q1 = 2 * Q1;
    Q2 = 2 * Q2;

    tmp8 = Q2 - eta.main * diag(diag(x.main' * Q1));
    output.main = (tmp8 - x.main * diag(diag(x.main' * tmp8)));
end