function test_MIMO_CPP_params()
    close all; clc;
    warning off matlab2tikz:deprecatedEnvironment;
    
    fprintf('Test MIMO Problem. \n');
    r = rand();
    r = floor(r * 10000);
    r = 1;
    rand('state', r);
    randn('state', r);
    
    % Brad's parameters
    fprintf('\n\nParameters: \n');
    Tw = 0.002;
    fprintf('Time: %f\n', Tw);
    W = 30000; % 30kHz;
    fprintf('Bandwidth: %d\n', W);
      
    % Compute time samples
    Tw = Tw/2;
    dt = 1/(4*W);
    fprintf('Time Samples: %f\n', dt);
    xt = -Tw:dt:Tw;
    fprintf('xt = %f : dt : %f\n', -Tw, Tw);
 
    % Number of different Riemannian algorithms
    nMethods = 8;
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     


    % Run for various number of signals
    for ns = 2 : 2 : 2
        % Increasing number of signals causes break down because
        % of signal-to-signal interference
        fprintf('\nNumber of Signals: %d \n', ns);
        if(ns == 2)
            % Compare with LFM signals
            compLFM = 1;
        else
            compLFM = 0;
        end
        
        % Run for various number of basis functions
        for nb = 50 : 5 : 50
            fprintf('\nNumber of Bases: %d \n', nb);
            
            % Manifold Parameters
            ManiParams.IsCheckParams = 0;
            ManiParams(1).name = 'Oblique';
            ManiParams(1).n = nb;
            ManiParams(1).m = ns;
            ManiParams(1).ParamSet = 2;
            
            if(ManiParams(1).ParamSet == 1)
                retract = sprintf('qfRetraction');
                vectTrans = sprintf('parallelization_LCnotsatisfied');
            elseif(ManiParams(1).ParamSet == 2)
                retract = sprintf('expMapping');
                vectTrans = sprintf('parallelTranslation_LCsatisfied');
            elseif(ManiParams(1).ParamSet == 3)
                retract = sprintf('qfRetraction');
                vectTrans = sprintf('parallelTranslation_LCnotsatisfied');
            elseif(ManiParams(1).ParamSet == 4)
                retract = sprintf('qfRetraction');
                vectTrans = sprintf('parallelTranslation_LCsatisfied');
            end
            
            % Run for various deltas
            for delta = 5 : 5 : 5
                fprintf('\nDelta: %d \n', delta);
                
                % Run multiple iterations
                for itr = 1 : 1
                    fprintf('\n\nRun %d\n\n', itr);


                    newDir = sprintf('newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d', ns, nb, Tw, W, delta, retract, vectTrans, itr);
                    mkdir('../images/cpp',newDir);

                    % Compute zero lag pswf functions
                    fn = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/pswf_func_time%0.0d_band%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, nb);
                    if exist(fn, 'file')
                        load(fn)
                    else
                        p0 = pswf(xt/Tw, Tw, W, nb).';
                        save(fn, 'p0');
                    end

                    % Compute correlation matrix
                    lag = -Tw:dt:Tw;
                    fprintf('lag = %f : dt : %f\n', -Tw, Tw);
                    flag = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/lags_time%0.0d_band%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, nb);
                    save(flag, 'lag');

                    maxLag = max(lag)/dt;
                    pad = ceil(maxLag);
                    p0new = [p0; zeros(pad, nb)];

                    nlags = length(lag);
                    C = zeros(nb, nb, nlags);
                    for l = 1 : nlags
                        pl = fourierShift(p0new.', lag(l), dt).';
                        C(:, :, l) = p0new' * pl;
                    end

                    % Initial Condtions

                    fx0 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/initialCondition_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, delta, nb);
                    if exist(fx0, 'file')
                        load(fx0);
                        x1 = x0;
                    else
                        x1 = randn(nb, ns);
                        [x1, ~] = qr(x1, 0);
                        x0 = x1;
                        save(fx0, 'x0');
                    end


                    InitialX.main = x1;
                    HasHHR = 0;
                    
                    fhandle = @(x)f(x, C, lag, delta);
                    gfhandle = @(x)gf(x, C, lag, delta);
                    Hesshandle = @(x, eta)Hess(x, eta, C, lag, delta);

                    tab = zeros(10, nMethods);
                    row = 0;
                    
                    % Solver Parameters
                    for method = 4:4 %1 : nMethods
                        if(method == 1)
                            alg = sprintf('RBFGS');
                            
                            SolverParams.method = 'RBFGS';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 50;
                            SolverParams.DEBUG = 2;
                            SolverParams.LineSearch_LS = 0;
                            SolverParams.LS_alpha = 1e-4;
                            SolverParams.LS_beta = 0.999;
                            SolverParams.Minstepsize = eps;
                            SolverParams.Maxstepsize = 1000;
                            SolverParams.LS_ratio = 0.25;
                            SolverParams.Initstepsize = 1;
                            SolverParams.nu = 1e-6;
                            SolverParams.mu = 1;
                        elseif(method == 2)
                            alg = sprintf('LRBFGS');
                            
                            SolverParams.method = 'LRBFGS';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 50;
                            SolverParams.DEBUG = 1;
                            SolverParams.LineSearch_LS = 0;
                            SolverParams.LS_alpha = 1e-4;
                            SolverParams.LS_beta = 0.999;
                            SolverParams.Minstepsize = eps;
                            SolverParams.Maxstepsize = 1000;
                            SolverParams.LS_ratio = 0.25;
                            SolverParams.Initstepsize = 1;
                            SolverParams.nu = 1e-6;
                            SolverParams.mu = 1;
                            SolverParams.LengthSY = 4;
                        elseif(method == 3)
                            alg = sprintf('RTR-SR1');
                            
                            SolverParams.method = 'RTRSR1';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 50;
                            SolverParams.DEBUG = 2;
                            SolverParams.Acceptence_Rho = 0.1;
                            SolverParams.Shrinked_tau = 0.25;
                            SolverParams.Magnified_tau = 2;
                            SolverParams.minimum_Delta = eps;
                            SolverParams.maximum_Delta = 1000;
                            SolverParams.useRand = 0;
                            SolverParams.Min_Inner_Iter = 0;
                            SolverParams.Max_Inner_Iter = 1000;
                            SolverParams.theta = 1;
                            SolverParams.kappa = 0.1;
                            SolverParams.initial_Delta = 1;
                        elseif(method == 4)
                            alg = sprintf('LRTR-SR1');
                            
                            SolverParams.method = 'LRTRSR1';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 1;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 10000;
                            SolverParams.OutputGap = 100;
                            SolverParams.DEBUG = 2;
                            SolverParams.Acceptence_Rho = 0.1;
                            SolverParams.Shrinked_tau = 0.25;
                            SolverParams.Magnified_tau = 2;
                            SolverParams.minimum_Delta = eps;
                            SolverParams.maximum_Delta = 1000;
                            SolverParams.useRand = 0;
                            SolverParams.Min_Inner_Iter = 0;
                            SolverParams.Max_Inner_Iter = 1000;
                            SolverParams.theta = 1;
                            SolverParams.kappa = 0.1;
                            SolverParams.initial_Delta = 1;
                            SolverParams.LengthSY = 4;
                        elseif(method == 5)
                            alg = sprintf('RSD');
                            
                            SolverParams.method = 'RSD';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 1;
                            SolverParams.DEBUG = 2;
                            SolverParams.LineSearch_LS = 0;
                            SolverParams.LS_alpha = 1e-4;
                            SolverParams.LS_beta = 0.999;
                            SolverParams.Minstepsize = eps;
                            SolverParams.Maxstepsize = 1000;
                            SolverParams.LS_ratio = 0.25;
                            SolverParams.Initstepsize = 1;
                        elseif(method == 6)
                            alg = sprintf('RNewton');
                            
                            SolverParams.method = 'RNewton';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 1;
                            SolverParams.DEBUG = 2;
                            SolverParams.LineSearch_LS = 0;
                            SolverParams.LS_alpha = 1e-4;
                            SolverParams.LS_beta = 0.999;
                            SolverParams.Minstepsize = eps;
                            SolverParams.Maxstepsize = 1000;
                            SolverParams.LS_ratio = 0.25;
                            SolverParams.Initstepsize = 1;
                            SolverParams.useRand = 0;
                            SolverParams.Min_Inner_Iter = 0;
                            SolverParams.Max_Inner_Iter = 1000;
                            SolverParams.theta = 1;
                            SolverParams.kappa = 0.1;
                        elseif(method == 7)
                            alg = sprintf('RTR-SD');
                            
                            SolverParams.method = 'RTRSD';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 1;
                            SolverParams.DEBUG = 2;
                            SolverParams.Acceptence_Rho = 0.1;
                            SolverParams.Shrinked_tau = 0.25;
                            SolverParams.Magnified_tau = 2;
                            SolverParams.minimum_Delta = eps;
                            SolverParams.maximum_Delta = 1000;
                            SolverParams.useRand = 0;
                            SolverParams.Min_Inner_Iter = 0;
                            SolverParams.Max_Inner_Iter = 1000;
                            SolverParams.theta = 1;
                            SolverParams.kappa = 0.1;
                            SolverParams.initial_Delta = 1;
                        elseif(method == 8)
                            alg = sprintf('RTR-Newton');
                            
                            SolverParams.method = 'RTRNewton';
                            SolverParams.IsCheckParams = 0;
                            SolverParams.IsCheckGradHess = 0;
                            SolverParams.Stop_Criterion = 2;
                            SolverParams.Tolerance = 1e-6;
                            SolverParams.Min_Iteration = 0;
                            SolverParams.Max_Iteration = 5000;
                            SolverParams.OutputGap = 1;
                            SolverParams.DEBUG = 2;
                            SolverParams.Acceptence_Rho = 0.1;
                            SolverParams.Shrinked_tau = 0.25;
                            SolverParams.Magnified_tau = 2;
                            SolverParams.minimum_Delta = eps;
                            SolverParams.maximum_Delta = 1000;
                            SolverParams.useRand = 0;
                            SolverParams.Min_Inner_Iter = 0;
                            SolverParams.Max_Inner_Iter = 1000;
                            SolverParams.theta = 1;
                            SolverParams.kappa = 0.1;
                            SolverParams.initial_Delta = 1;
                        end
                        currDir = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d', ns, nb, Tw, W, delta, retract, vectTrans, itr);
                        mkdir(currDir, alg);

                        % Run Optimization Problem 
                        times = 0;
                        total_timecost = 0;
                        T = 0;
                        X = 0; Fv = 0; GfG0 = 0; iter = 0; nf = 0; ng = 0; 
                        nR = 0; nV = 0; nVp = 0; nH = 0; ComTime = 0;
                        funs = []; grad = []; accumTimes = [];
                        while(times < 1)
                            fprintf('%d: ', times);
                            [X, Fv, Gfv, GfG0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grad, accumTimes] = DriverOPT(fhandle, gfhandle, Hesshandle, SolverParams, ManiParams, HasHHR, InitialX);
                            if(times == 0)
                                T = accumTimes;
                            else
                                T = T + accumTimes;
                            end

                            total_timecost = total_timecost + ComTime;
                            times = times + 1;
                        end
                        T = T / times;
                        ave_time = total_timecost / times;
                        fprintf('average time cost of %d times is %f. \n', times, ave_time);

                        tab(row + 1, method) = iter;
                        tab(row + 2, method) = nf;
                        tab(row + 3, method) = ng;
                        tab(row + 4, method) = nH;
                        tab(row + 5, method) = nV + nVp;
                        tab(row + 6, method) = nR;
                        tab(row + 7, method) = Fv;
                        tab(row + 8, method) = Gfv;
                        tab(row + 9, method) = GfG0;
                        tab(row + 10, method) = ave_time;
%                         MIMO_FS{method} = funs;
                        MIMO_GS{method} = grad;
%                         MIMO_GFG0S{method} = grad/grad(1);
                        MIMO_TS{method} = T;

                        S = p0 * X.main;
                        [n, m] = size(S);
                        fb = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/coef_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W, delta, nb);
                        B = X.main;
                        save(fb, 'B');
                        fs = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/signal_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W, delta, nb);
                        save(fs, 'S');

                        imgFileEps1 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/signals_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W, delta, nb);
                        imgFilePng1 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/signals_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W, delta, nb);
                        imgFileFig1 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/signals_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W, delta, nb);
                        imgFileTex1 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/signals_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W, delta, nb);


%                         if(compLFM == 1)
%                             % Signals
%                             sigTime = (1:n)*dt;
%                             h = figure(1);clf
%                             hs1 = subplot(2, 1, 1);
%                             plot(sigTime, S(:, 1), 'b')
%                             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                             xlabel('time', 'fontsize',14); 
%                             ylabel('s_1', 'fontsize',14)
% 
%                             hs2 = subplot(2, 1, 2);
%                             plot(sigTime, S(:, 2), 'b')
%                             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                             xlabel('time','fontsize',14); 
%                             ylabel('s_2','fontsize',14)
% 
%                             ylb = min(min(S, [], 1));
%                             yub = max(max(S, [], 1));
%                             axis([hs1 hs2], [sigTime(1) sigTime(n) ylb yub]);
% 
%                             saveas(h, imgFileEps1, 'psc2');
%                             saveas(h, imgFilePng1);
%                             saveas(h, imgFileFig1);
%                             
%                             cleanfigure;
%                             matlab2tikz(imgFileTex1, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                             close(h)
% 
%                             % LFM Signals
%                             h = figure(20);clf
%                             [s1, s2] = mimo_lfm(2*Tw, W, dt);
%                             [n1, m1] = size(s1);
%                             time1 = (1:m1)*dt;
%                             hs1 = subplot(2, 1, 1);
%                             plot(time1, s1, 'r')
%                             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                             xlabel('time','fontsize',14); 
%                             ylabel('s_1','fontsize',14)
% 
%                             hs2 = subplot(2, 1, 2);
%                             plot(time1, s2, 'r')
%                             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                             xlabel('time','fontsize',14); 
%                             ylabel('s_2','fontsize',14)
%                             ylb = min(min(s1, s2));
%                             yub = max(max(s1, s2));
%                             axis([hs1 hs2], [time1(1) time1(m1) ylb yub]);
% 
%                             imgFileEps20 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/LFMsignals_time%0.0d_band%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W);
%                             imgFilePng20 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/LFMsignals_time%0.0d_band%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W);
%                             imgFileFig20 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/LFMsignals_time%0.0d_band%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W);
%                             imgFileTex20 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/LFMsignals_time%0.0d_band%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, Tw, W);
% 
%                             saveas(h, imgFileEps20, 'psc2');
%                             saveas(h, imgFilePng20);
%                             saveas(h, imgFileFig20);
% 
%                             cleanfigure;
%                             matlab2tikz(imgFileTex20, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                             close(h);
%                             for nsi = 1 : ns
%                                 imgFileEps2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 imgFilePng2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 imgFileFig2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 imgFileTex2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 % Auto-Correltaion
%                                 h = figure(2);clf
%                                 hold on;
%                                 [acf2, aclags] = xcorr(S(:, nsi), floor(n/2));
%                                 plot(aclags*dt, 20*log10(abs(acf2)))
%                                 axis([-Tw Tw -30 0])
%                                 set(gca,'fontsize',14)
% 
%                                 if(nsi == 1)
%                                     [acf2, aclags] = xcorr(s1, floor(m1/2));
%                                     plot(aclags*dt, 20*log10(abs(acf2)), '--r')
%                                 elseif(nsi == 2)
%                                     [acf2, aclags] = xcorr(s2, floor(m1/2));
%                                     plot(aclags*dt, 20*log10(abs(acf2)), '--r')
%                                 else
%                                     error('Cannot compute more that two LFM signals!');
%                                 end
% 
%                                 imgTitle = sprintf('auto-correlation of s_{%d}', nsi);
%                                 title(imgTitle,'fontsize',14)
%                                 xlabel('\tau time lag(s)','fontsize',14);
%                                 imgYLABEL = sprintf('20log_{10}(|r_{%d,%d}(\tau)|)', nsi, nsi);
%                                 ylabel(imgYLABEL, 'fontsize',14);
% 
%                                 autoCorr = sprintf('%s', alg);
%                                 legend(autoCorr, 'LFM', 'Location', 'NorthEast');
%                                 hold off;
% 
%                                 saveas(h, imgFileEps2, 'psc2');
%                                 saveas(h, imgFilePng2);
%                                 saveas(h, imgFileFig2);
% 
%                                 cleanfigure;
%                                 matlab2tikz(imgFileTex2, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                                 close(h)
% 
%                                 for nsj = nsi+1 : ns
%                                     imgFileEps3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     imgFilePng3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     imgFileFig3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     imgFileTex3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     % Cross-Correlation
%                                     h = figure(3);clf
%                                     hold on;
%                                     [ccf, clags] = xcorr(S(:, nsi), S(:, nsj), floor(n/2));
%                                     plot(clags*dt, 20*log10(abs(ccf)))
%                                     axis([-Tw Tw min(20*log10(abs(ccf))) 0])
%                                     set(gca,'fontsize',14)
% 
%                                     [ccf, clags] = xcorr(s1, s2, floor(m1/2));
%                                     plot(clags*dt, 20*log10(abs(ccf)), '--r')
% 
%                                     imgTitle = sprintf('cross-correlation between s_{%d} and s_{%d}', nsi, nsj);
%                                     title(imgTitle,'fontsize',14)
%                                     xlabel('\tau time lag(s)','fontsize',14);
%                                     imgYLABEL = sprintf('20log_{10}(|r_{%d,%d}(\tau)|)', nsi, nsj);
%                                     ylabel(imgYLABEL, 'fontsize',14);
%                                     xCorrS1S2 = sprintf('%s', alg);
%                                     legend(xCorrS1S2, 'LFM', 'Location', 'NorthEast');
%                                     hold off;
% 
%                                     saveas(h, imgFileEps3, 'psc2');
%                                     saveas(h, imgFilePng3);
%                                     saveas(h, imgFileFig3);
% 
%                                     cleanfigure;
%                                     matlab2tikz(imgFileTex3, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                                     close(h)
%                                 end
%                             end
%                         else
%                             % Signals
%                             sigTime = (1:n)*dt;
%                             h = figure(1);clf
%                             
%                             hold on;
%                             hs = zeros(ns, 1);
%                             nSubCol = ns / 2;
%                             for i = 1 : ns       
%                                 hs(i) = subplot(2, nSubCol, i);
%                                 plot(sigTime, S(:, i), 'b')
%                                 set(get(gcf,'CurrentAxes'),'fontsize',14)
%                                 
%                                 imgYLABEL = sprintf('s__{%d}', i);
%                                 xlabel('time', 'fontsize', 14); 
%                                 ylabel(imgYLABEL, 'fontsize', 14)
%                             end
% 
% %                             hs2 = subplot(2, 2, 2);
% %                             plot(sigTime,S(:, 2), 'b')
% %                             set(get(gcf,'CurrentAxes'),'fontsize',14)
% %                             xlabel('time','fontsize',14); 
% %                             ylabel('s_2','fontsize',14)
% % 
% %                             hs3 = subplot(2, 2, 3);
% %                             plot(sigTime,S(:,3),'b')
% %                             set(get(gcf,'CurrentAxes'),'fontsize',14)
% %                             xlabel('time','fontsize',14); ylabel('s_3','fontsize',14)
% % 
% %                             hs4 = subplot(2, 2, 4);
% %                             plot(sigTime,S(:,4),'b')
% %                             set(get(gcf,'CurrentAxes'),'fontsize',14)
% %                             xlabel('time','fontsize',14); ylabel('s_4','fontsize',14)
% % 
%                         %             hs5 = subplot(2, 4, 5);
%                         %             plot(sigTime,S(:,5),'b')
%                         %             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                         %             xlabel('time','fontsize',14); ylabel('s_5','fontsize',14)
%                         %             
%                         %             hs6 = subplot(2, 4, 6);
%                         %             plot(sigTime,S(:,6),'b')
%                         %             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                         %             xlabel('time','fontsize',14); ylabel('s_6','fontsize',14)
%                         %             
%                         %             hs7 = subplot(2, 4, 7);
%                         %             plot(sigTime,S(:,7),'b')
%                         %             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                         %             xlabel('time','fontsize',14); ylabel('s_7','fontsize',14)
%                         %             
%                         %             hs8 = subplot(2, 4, 8);
%                         %             plot(sigTime,S(:,8),'b')
%                         %             set(get(gcf,'CurrentAxes'),'fontsize',14)
%                         %             xlabel('time','fontsize',14); ylabel('s_8','fontsize',14)
% 
%                             ylb = min(min(S, [], 1));
%                             yub = max(max(S, [], 1));
%                             axis([hs], [sigTime(1) sigTime(n) ylb yub]);
% 
%                             saveas(h, imgFileEps1, 'psc2');
%                             saveas(h, imgFilePng1);
%                             saveas(h, imgFileFig1);
% 
%                             cleanfigure;
%                             matlab2tikz(imgFileTex1, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                             close(h)
% 
%                             for nsi = 1 : ns
%                                 imgFileEps2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 imgFilePng2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 imgFileFig2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
%                                 imgFileTex2 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, Tw, W, delta, nb);
% 
%                                 % Auto-Correltaion
%                                 h = figure(2);clf
%                                 [acf2, aclags] = xcorr(S(:, nsi), floor(n/2));
%                                 plot(aclags*dt, 20*log10(abs(acf2)))
%                                 axis([-Tw Tw -30 0])
%                                 set(gca,'fontsize',14)
%                                 imgTitle = sprintf('auto-correlation of s_{%d}', nsi);
%                                 title(imgTitle,'fontsize',14)
%                                 xlabel('\tau time lag(s)','fontsize',14);
%                                 imgYLABEL = sprintf('20log_{10}(|r_{%d,%d}(\tau)|)', nsi, nsi);
%                                 ylabel(imgYLABEL, 'fontsize',14);
% 
%                                 saveas(h, imgFileEps2, 'psc2');
%                                 saveas(h, imgFilePng2);
%                                 saveas(h, imgFileFig2);
% 
%                                 cleanfigure;
%                                 matlab2tikz(imgFileTex2, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                                 close(h)
% 
%                                 for nsj = nsi+1 : ns
%                                     imgFileEps3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     imgFilePng3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     imgFileFig3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
%                                     imgFileTex3 = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, alg, nsi, nsj, Tw, W, delta, nb);
% 
%                                     % Cross-Correlation
%                                     h = figure(3);clf
%                                     [ccf, clags] = xcorr(S(:, nsi), S(:, nsj), floor(n/2));
%                                     plot(clags*dt, 20*log10(abs(ccf)))
%                                     axis([-Tw Tw min(20*log10(abs(ccf))) 0])
%                                     set(gca,'fontsize',14)
%                                     imgTitle = sprintf('cross-correlation between s_{%d} and s_{%d}', nsi, nsj);
%                                     title(imgTitle,'fontsize',14)
%                                     xlabel('\tau time lag(s)','fontsize',14);
%                                     imgYLABEL = sprintf('20log_{10}(|r_{%d,%d}( \tau )|)', nsi, nsj);
%                                     ylabel(imgYLABEL, 'fontsize',14);
% 
%                                     saveas(h, imgFileEps3, 'psc2');
%                                     saveas(h, imgFilePng3);
%                                     saveas(h, imgFileFig3);
% 
%                                     cleanfigure;
%                                     matlab2tikz(imgFileTex3, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                                     close(h)
%                                 end
%                             end
%                         end
                    end
% 
%                     % Table
%                     fileName = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/MIMO_run%d_tab.txt', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr);
%                     fout = fopen(fileName,'w+');
%                     for i = 1 : 10
%                         for j = 1 : 8
%                             if(mod(i-1, 10) < 6)
%                                 if(j < nMethods)
%                                     fprintf(fout,'%d & ',tab(i, j));
%                                 else
%                                     fprintf(fout,'%d', tab(i, j));
%                                 end
%                             else
%                                 tabstr = outputfloat(tab(i, j));
%                                 if(j < nMethods)
%                                     fprintf(fout,['$' tabstr '$ & ']);
%                                 else
%                                     fprintf(fout,['$' tabstr '$']);
%                                 end
%                             end
%                         end
%                         fprintf(fout,' \\\\\n');
%                     end
%                     fclose(fout);
% 
%                     fMIMO_FS = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/allAlgs_functionVals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, delta, nb);
%                     fMIMO_GS = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/allAlgs_normGradientVals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, delta, nb);
%                     fMIMO_GFG0S = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/allAlgs_normGradfGrad0Vals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, delta, nb);
%                     fMIMO_TS = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/allAlgs_TimeVals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, itr, Tw, W, delta, nb);
%                     save(fMIMO_FS, 'MIMO_FS');
%                     save(fMIMO_GS, 'MIMO_GS');
%                     save(fMIMO_GFG0S, 'MIMO_GFG0S');
%                     save(fMIMO_TS, 'MIMO_TS');
% 
%                     % Gradient: Algorithm Iteration and Time Comparison
%                     h = figure('Position', [1, 1, 720, 480]);clf
%                     set(0,'defaultaxesfontsize',10, ...
%                        'defaultaxeslinewidth',0.7, ...
%                        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%                     set(0,'defaultlinemarkersize',10)
%                     col = num2cell(jet(8),2);
% 
%             %         subplot(2, 1, 1)
%                     for i = 1 : nMethods
%                         p = semilogy(1 : length(MIMO_GS{i}), MIMO_GS{i}, '.');
%                         set(p, 'color' ,col{i});
%                         hold on
%                     end
%                     axis square
%                     axis fill
%                     axis tight
%                     xlabel('iter.')
%                     ylabel('|grad f|')
%             %         title('MIMO');
% 
%                     if(nMethods == 1)
%                         legend('RBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 2)
%                         legend('RBFGS', 'LRBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 3)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 4)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 5)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 6)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 7)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 8)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'RTR-Newton', 'Location', 'NorthEastOutside');
%                     end
%                     algFileEps = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_iter_run%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFilePng = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_iter_run%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileFig = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_iter_run%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileTex = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_iter_run%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     saveas(h, algFileEps, 'psc2');
%                     saveas(h, algFilePng);
%                     saveas(h, algFileFig);
%                     axoptions={'ylabel={$\lvert$ grad f $\rvert$}'}; 
%                     cleanfigure;
%                     matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
%                     close(h)
% 
% 
%                     h = figure('Position', [1, 1, 720, 480]);clf
%                     set(0,'defaultaxesfontsize',10, ...
%                        'defaultaxeslinewidth',0.7, ...
%                        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%                     set(0,'defaultlinemarkersize',10)
%                     col = num2cell(jet(8),2);
% 
%             %         subplot(2, 1, 2)
%                     for i = 1 : nMethods
%                         p = semilogy(MIMO_TS{i}, MIMO_GS{i}, '.');
%                         set(p, 'color' ,col{i})
%                         hold on
%                     end
%                     axis square
%                     axis fill
%                     axis tight
%                     xlabel('time(second)')
%                     ylabel('|grad f|')
%             %         title('MIMO');  
% 
%                     if(nMethods == 1)
%                         legend('RBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 2)
%                         legend('RBFGS', 'LRBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 3)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 4)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 5)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 6)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 7)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 8)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'RTR-Newton', 'Location', 'NorthEastOutside');
%                     end
%                     algFileEps = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_time_run%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFilePng = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_time_run%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileFig = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_time_run%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileTex = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_grad_time_run%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);        
%                     saveas(h, algFileEps, 'psc2');
%                     saveas(h, algFilePng);
%                     saveas(h, algFileFig);
%                     axoptions={'ylabel={$\lvert$ grad f $\rvert$}'}; 
%                     cleanfigure;
%                     matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
%                     close(h)
% 
%                     % Gradient: Algorithm Iteration and Time Comparison
%                     h = figure('Position', [1, 1, 720, 480]);clf
%                     set(0,'defaultaxesfontsize',10, ...
%                        'defaultaxeslinewidth',0.7, ...
%                        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%                     set(0,'defaultlinemarkersize',10)
%                     col = num2cell(jet(8),2);
% 
%             %         subplot(2, 1, 1)
%                     for i = 1 : nMethods
%                         p = semilogy(1 : length(MIMO_GFG0S{i}), MIMO_GFG0S{i}, '.');
%                         set(p, 'color' ,col{i});
%                         hold on
%                     end
%                     axis square
%                     axis fill
%                     axis tight
%                     xlabel('iter.')
%                     ylabel('|grad f| / |grad f_0|')
%             %         title('MIMO');
% 
%                     if(nMethods == 1)
%                         legend('RBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 2)
%                         legend('RBFGS', 'LRBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 3)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 4)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 5)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 6)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 7)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 8)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'RTR-Newton', 'Location', 'NorthEastOutside');
%                     end
%                     algFileEps = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_iter_run%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFilePng = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_iter_run%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileFig = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_iter_run%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileTex = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_iter_run%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     saveas(h, algFileEps, 'psc2');
%                     saveas(h, algFilePng);
%                     saveas(h, algFileFig);
%                     axoptions={'ylabel={$\frac{\lvert\text{ grad f }\rvert}{\lvert\text{ grad f$_0$ }\rvert}$}'}; 
%                     cleanfigure;
%                     matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
%                     close(h)
% 
% 
%                     h = figure('Position', [1, 1, 720, 480]);clf
%                     set(0,'defaultaxesfontsize',10, ...
%                        'defaultaxeslinewidth',0.7, ...
%                        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%                     set(0,'defaultlinemarkersize',10)
%                     col = num2cell(jet(8),2);
% 
%             %         subplot(2, 1, 2)
%                     for i = 1 : nMethods
%                         p = semilogy(MIMO_TS{i}, MIMO_GFG0S{i}, '.');
%                         set(p, 'color' ,col{i})
%                         hold on
%                     end
%                     axis square
%                     axis fill
%                     axis tight
%                     xlabel('time(second)')
%                     ylabel('|grad f| / |grad f_0|')
%             %         title('MIMO');  
% 
%                     if(nMethods == 1)
%                         legend('RBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 2)
%                         legend('RBFGS', 'LRBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 3)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 4)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 5)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 6)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 7)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 8)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'RTR-Newton', 'Location', 'NorthEastOutside');
%                     end
%                     algFileEps = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_time_run%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFilePng = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_time_run%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileFig = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_time_run%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileTex = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_gradfgrad0_time_run%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);        
%                     saveas(h, algFileEps, 'psc2');
%                     saveas(h, algFilePng);
%                     saveas(h, algFileFig);
%                     axoptions={'ylabel={$\frac{\lvert\text{ grad f }\rvert}{\lvert\text{ grad f$_0$ }\rvert}$}'};
%                     cleanfigure;
%                     matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
%                     close(h)
% 
%                     % Function: Algorithm Iteration and Time Comparison
%                     h = figure('Position', [1, 1, 720, 480]);clf
%                     set(0,'defaultaxesfontsize',10, ...
%                        'defaultaxeslinewidth',0.7, ...
%                        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%                     set(0,'defaultlinemarkersize',10)
%                     col = num2cell(jet(8),2);
% 
%             %         subplot(2, 1, 1)
%                     for i = 1 : nMethods
%                         p = semilogy(1 : length(MIMO_FS{i}), MIMO_FS{i}, '.');
%                         set(p, 'color' ,col{i});
%                         hold on
%                     end
%                     axis square
%                     axis fill
%                     axis tight
%                     xlabel('iter.')
%                     ylabel('f')
%             %         title('MIMO');
% 
%                     if(nMethods == 1)
%                         legend('RBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 2)
%                         legend('RBFGS', 'LRBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 3)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 4)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 5)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 6)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 7)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 8)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'RTR-Newton', 'Location', 'NorthEastOutside');
%                     end
%                     algFileEps = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_iter_run%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFilePng = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_iter_run%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileFig = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_iter_run%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileTex = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_iter_run%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     saveas(h, algFileEps, 'psc2');
%                     saveas(h, algFilePng);
%                     saveas(h, algFileFig);
%                     cleanfigure;
%                     matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                     close(h)
% 
% 
%                     h = figure('Position', [1, 1, 720, 480]);clf
%                     set(0,'defaultaxesfontsize',10, ...
%                        'defaultaxeslinewidth',0.7, ...
%                        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%                     set(0,'defaultlinemarkersize',10)
%                     col = num2cell(jet(8),2);
% 
%             %         subplot(2, 1, 2)
%                     for i = 1 : nMethods
%                         p = semilogy(MIMO_TS{i}, MIMO_FS{i}, '.');
%                         set(p, 'color' ,col{i})
%                         hold on
%                     end
%                     axis square
%                     axis fill
%                     axis tight
%                     xlabel('time(second)')
%                     ylabel('f')
%             %         title('MIMO');  
% 
%                     if(nMethods == 1)
%                         legend('RBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 2)
%                         legend('RBFGS', 'LRBFGS', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 3)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 4)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 5)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 6)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 7)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'Location', 'NorthEastOutside');
%                     elseif(nMethods == 8)
%                         legend('RBFGS', 'LRBFGS', 'RTR-SR1', 'LRTR-SR1', 'RSD', 'RNewton', 'RTR-SD', 'RTR-Newton', 'Location', 'NorthEastOutside');
%                     end
%                     algFileEps = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_time_run%d_time%0.0d_band%d_delta%d_nb%d.eps', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFilePng = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_time_run%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileFig = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_time_run%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);
%                     algFileTex = sprintf('../images/cpp/newcostfunc_%dsignals_nb%d_time%f_band%d_delta%d_avg/%s/%s/run_%d/algs_func_time_run%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, itr, itr, Tw, W, delta, nb);        
%                     saveas(h, algFileEps, 'psc2');
%                     saveas(h, algFilePng);
%                     saveas(h, algFileFig);
%                     cleanfigure;
%                     matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
%                     close(h)
                end
            end
        end
    end
end


function str = outputfloat(x)
    if(x <= 0)
        str = '';
        return;
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [strx '_{' num2str(p) '}'];
    else
        str = strx;
    end
end

%% Objects
function [output, x] = f(x, C, lag, delta)
    % Cost Function
%     [nb,  ns] = size(x.main);
    N = size(C, 3);
    I1 = 0;
    
%     x.CU = zeros(nb, ns);
%     x.UtCU = zeros(ns, ns, N);
    
    % Neighborhood around tau = 0
    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    a = lag(1); b = lag(dl);
    for i = 1 : dl
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        f = norm(tmp2,'fro')^2;
        
        if(i == 1 || i == dl)
            I1 = I1 + ((b - a) / (N - 1)) * (f / 2);
        else
            I1 = I1 + ((b - a) / (N - 1)) * f;
        end
    end
    
    a = lag(dl); b = lag(du);
    for i = dl : du
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        fO = norm((triu(tmp2,1) + tril(tmp2,-1)),'fro')^2;
        
        if(i == dl || i == du)
            I1 = I1 + ((b - a) / (N - 1)) * (fO / 2);
        else
            I1 = I1 + ((b - a) / (N - 1)) * fO;
        end
    end
    
    a = lag(du); b = lag(N);
    for i = du : N
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        f = norm(tmp2,'fro')^2;
        
        if(i == du || i == N)
            I1 = I1 + ((b - a) / (N - 1)) * (f / 2);
        else
            I1 = I1 + ((b - a) / (N - 1)) * f;
        end
    end
    output = I1;
end

function [output, x] = gf(x, C, lag, delta)
    % Euclidean Gradient
    [nb, ns] = size(x.main);
    N = size(C, 3);
    Q1 = zeros(nb, ns);
    
    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    
%     x.CtU = zeros(nb, ns, N);
    
%     if(~isfield(x, 'CU'))
%         for i = 1 : N
%             x.CU(:, :, i) = C(:, :, i) * x.main;
%         end
%     end
%     
%     if(~isfield(x, 'UtCU'))
%         for i = 1 : N
%             x.UtCU(:, :, i) = x.main' * x.CU(:, :, i);
%         end
%     end
    
    a = lag(1); b = lag(dl);
    for i = 1 : dl
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        gf = (tmp1 * tmp2') + (tmp3 * tmp2);
        
        if(i == 1 || i == dl)
            Q1 = Q1 + ((b - a) / (N - 1)) * (gf / 2);
        else
            Q1 = Q1 + ((b - a) / (N - 1)) * gf;
        end
    end
    
    a = lag(dl); b = lag(du);
    for i = dl : du
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        gfO = (tmp1 * (triu(tmp2 ,1) + tril(tmp2 ,-1))') + (tmp3 * (triu(tmp2,1) + tril(tmp2,-1)));
        
        if(i == dl || i == du)
            Q1 = Q1 + ((b - a) / (N - 1)) * (gfO / 2);
        else
            Q1 = Q1 + ((b - a) / (N - 1)) * gfO;
        end
    end
    
    a = lag(du); b = lag(N);
    for i = du : N
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        gf = (tmp1 * tmp2') + (tmp3 * tmp2);
        
        if(i == du || i == N)
            Q1 = Q1 + ((b - a) / (N - 1)) * (gf / 2);
        else
            Q1 = Q1 + ((b - a) / (N - 1)) * gf;
        end
    end

    output.main = 2 * Q1;
end

function [output, x] = Hess(x, eta, C, lag, delta)
    % Euclidean Hessian
    [nb, ns] = size(x.main);
    N = size(C, 3);
    Q2 = zeros(nb, ns);

    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
%     if(~isfield(x, 'CU'))
%         for i = 1 : N
%             x.CU(:, :, i) = C(:, :, i) * x.main;
%         end
%     end
%     
%     if(~isfield(x, 'UtCU'))
%         for i = 1 : N
%             x.UtCU(:, :, i) = x.main' * x.CU(:, :, i);
%         end
%     end
%     
%     if(~isfield(x, 'CtU'))
%         for i = 1 : N
%             x.CtU(:, :, i) = C(:, :, i)' * x.main;
%         end
%     end
    
    a = lag(1); b = lag(dl);
    for i = 1 : dl
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * tmp1;
        tmp7 = x.main' * tmp4;
        
        Dgf = (tmp4 * tmp2') + (tmp1 * tmp6') + (tmp1 * tmp7') + ...
            (tmp5 * tmp2) + (tmp3 * tmp6) + (tmp3 * tmp7);
        
        if(i == 1 || i == dl)
            Q2 = Q2 + ((b - a) / (N-1)) * (Dgf / 2);
        else
            Q2 = Q2 + ((b - a) / (N-1)) * Dgf;
        end
    end
    
    a = lag(dl); b = lag(du);
    for i = dl : du
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * tmp1;
        tmp7 = x.main' * tmp4;
        
        DgfO = (tmp4 * (triu(tmp2,1) + tril(tmp2,-1))') + (tmp1 * (triu(tmp6,1) + tril(tmp6,-1))') + ...
            (tmp1 * (triu(tmp7,1) + tril(tmp7,-1))') + (tmp5 * (triu(tmp2,1) + tril(tmp2,-1))) + ...
            (tmp3 * (triu(tmp6,1) + tril(tmp6,-1))) + (tmp3 * (triu(tmp7,1) + tril(tmp7,-1)));

        if(i == dl || i == du)
            Q2 = Q2 + ((b - a) / (N-1)) * (DgfO / 2);
        else
            Q2 = Q2 + ((b - a) / (N-1)) * DgfO;
        end
    end
    
    a = lag(du); b = lag(N);
    for i = du : N
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * tmp1;
        tmp7 = x.main' * tmp4;
        
        Dgf = (tmp4 * tmp2') + (tmp1 * tmp6') + (tmp1 * tmp7') + ...
            (tmp5 * tmp2) + (tmp3 * tmp6) + (tmp3 * tmp7);
        
        if(i == du || i == N)
            Q2 = Q2 + ((b - a) / (N-1)) * (Dgf / 2);
        else
            Q2 = Q2 + ((b - a) / (N-1)) * Dgf;
        end
    end 
    output.main = 2 * Q2;
end